#include "MPU6050_acq_sf.h"
#include "MPU6050_acq_sf_private.h"
#include "rtwtypes.h"
#include "limits.h"
#include "board.h"
#include "mw_cmsis_rtos.h"
#define UNUSED(x)                      x = x
#define EXIT_ON_ERROR(msg, cond)       if (cond) { return(0); }

extern const char *TgtConnInit(int_T argc, char_T *argv[]);
extern void TgtConnTerm();
extern void TgtConnPreStep(int_T tid);
extern void TgtConnPostStep(int_T tid);
const char * csErrorStatus;

/* Function prototype declaration*/
void exitFcn(int sig);
void *terminateTask(void *arg);
void *baseRateTask(void *arg);
void *subrateTask(void *arg);
volatile boolean_T runModel = true;
mw_signal_event_t stopSem;
mw_signal_event_t baserateTaskSem;
mw_thread_t schedulerThread;
mw_thread_t baseRateThread;
mw_thread_t backgroundThread;
unsigned long threadJoinStatus[8];
int terminatingmodel = 0;
void *baseRateTask(void *arg)
{
  runModel = (rtmGetErrorStatus(MPU6050_acq_sf_M) == (NULL)) &&
    !rtmGetStopRequested(MPU6050_acq_sf_M);
  while (runModel) {
    mw_osSignalEventWaitEver(&baserateTaskSem);

    /* External mode */
    {
      boolean_T rtmStopReq = false;
      rtExtModePauseIfNeeded(MPU6050_acq_sf_M->extModeInfo, 1, &rtmStopReq);
      if (rtmStopReq) {
        rtmSetStopRequested(MPU6050_acq_sf_M, true);
      }

      if (rtmGetStopRequested(MPU6050_acq_sf_M) == true) {
        rtmSetErrorStatus(MPU6050_acq_sf_M, "Simulation finished");
        break;
      }
    }

    MPU6050_acq_sf_step();

    /* Get model outputs here */
    rtExtModeCheckEndTrigger();
    runModel = (rtmGetErrorStatus(MPU6050_acq_sf_M) == (NULL)) &&
      !rtmGetStopRequested(MPU6050_acq_sf_M);
  }

  runModel = 0;
  terminateTask(arg);
  mw_osThreadExit((void *)0);
  return NULL;
}

void exitFcn(int sig)
{
  UNUSED(sig);
  rtmSetErrorStatus(MPU6050_acq_sf_M, "stopping the model");
}

void *terminateTask(void *arg)
{
  UNUSED(arg);
  terminatingmodel = 1;

  {
    runModel = 0;

    /* Wait for background task to complete */
    CHECK_STATUS(mw_osThreadJoin(backgroundThread,(void *)&threadJoinStatus), 0,
                 "mw_osThreadJoin");
  }

  rtExtModeShutdown(1);
  TgtConnTerm();

  /* Disable rt_OneStep() here */

  /* Terminate model */
  MPU6050_acq_sf_terminate();
  mw_osSignalEventRelease(&stopSem);
  return NULL;
}

void backgroundTask(void *arg)
{
  while (runModel) {
    /* External mode */
    {
      boolean_T rtmStopReq = false;
      rtExtModeOneStep(MPU6050_acq_sf_M->extModeInfo, 1, &rtmStopReq);
      if (rtmStopReq) {
        rtmSetStopRequested(MPU6050_acq_sf_M, true);
      }
    }

    runCommService();
  }
}

int main(int argc, char **argv)
{
  SystemCoreClockUpdate();
  hardware_init();
  rtmSetErrorStatus(MPU6050_acq_sf_M, 0);
  rtExtModeParseArgs(argc, (const char_T **)argv, NULL);

  /* Initialize model */
  MPU6050_acq_sf_initialize();

  /* Target connectivity initialization */
  csErrorStatus = TgtConnInit(argc, argv);
  EXIT_ON_ERROR("Error initializing target connectivity: %s\n", csErrorStatus);

  /* External mode */
  rtSetTFinalForExtMode(&rtmGetTFinal(MPU6050_acq_sf_M));
  rtExtModeCheckInit(1);

  {
    boolean_T rtmStopReq = false;
    rtExtModeWaitForStartPkt(MPU6050_acq_sf_M->extModeInfo, 1, &rtmStopReq);
    if (rtmStopReq) {
      rtmSetStopRequested(MPU6050_acq_sf_M, true);
    }
  }

  rtERTExtModeStartMsg();

  /* Call RTOS Initialization function */
  mw_RTOSInit(0.05, 0);

  /* Wait for stop semaphore */
  mw_osSignalEventWaitEver(&stopSem);
  return 0;
}
